!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ##################
      MODULE MODD_DATA_TEB_GARDEN_n
!     ##################
!
!!****  *MODD_DATA_ISBA - declaration of DATA surface parameters for ISBA scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson  *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       05/2005
!
!*       0.   DECLARATIONS
!             ------------
!
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DATA_TEB_GARDEN_t
!-------------------------------------------------------------------------------
!
  REAL, POINTER, DIMENSION(:)   :: XDATA_FRAC_HVEG        ! fraction of high vegetation
  REAL, POINTER, DIMENSION(:)   :: XDATA_FRAC_LVEG        ! fraction of low  vegetation
  REAL, POINTER, DIMENSION(:)   :: XDATA_FRAC_NVEG        ! fraction of bare soil
  REAL, POINTER, DIMENSION(:,:) :: XDATA_LAI_HVEG         ! LAI      of high vegetation
  REAL, POINTER, DIMENSION(:,:) :: XDATA_LAI_LVEG         ! LAI      of low  vegetation
  REAL, POINTER, DIMENSION(:)   :: XDATA_H_HVEG           ! height of trees
!
!
! Mask and number of grid elements containing patches/tiles:
!
  REAL, POINTER, DIMENSION(:,:)  :: XDATA_VEGTYPE       ! fraction of each vegetation type for
!                                                  ! each grid mesh                          (-)
!
!-------------------------------------------------------------------------------
!
  INTEGER                       :: NTIME               ! number of time data
!                                                      ! for VEG, LAI, EMIS, Z0
!
! Input Parameters:
!
! - vegetation + bare soil:
!
  REAL, POINTER, DIMENSION(:) :: XDATA_Z0_O_Z0H         ! ratio of surface roughness lengths
!                                                      ! (momentum to heat)                      (-)
  REAL, POINTER, DIMENSION(:,:) :: XDATA_EMIS             ! surface emissivity                      (-)
  REAL, POINTER, DIMENSION(:,:) :: XDATA_Z0               ! surface roughness length                (m)
!
! - vegetation:
!
  REAL, POINTER, DIMENSION(:) :: XDATA_ALBNIR_VEG       ! vegetation near-infra-red albedo        (-)
  REAL, POINTER, DIMENSION(:) :: XDATA_ALBVIS_VEG       ! vegetation visible albedo               (-)
  REAL, POINTER, DIMENSION(:) :: XDATA_ALBUV_VEG        ! vegetation UV albedo                    (-)
!
! - vegetation: default option (Jarvis) and general parameters:
!
  REAL, POINTER, DIMENSION(:,:) :: XDATA_VEG            ! vegetation cover fraction               (-)
  REAL, POINTER, DIMENSION(:) :: XDATA_WRMAX_CF         ! coefficient for maximum water 
!                                                      ! interception 
!                                                      ! storage capacity on the vegetation      (-)
  REAL, POINTER, DIMENSION(:) :: XDATA_RSMIN            ! minimum stomatal resistance             (s/m)
  REAL, POINTER, DIMENSION(:) :: XDATA_GAMMA            ! coefficient for the calculation
!                                                      ! of the surface stomatal
!                                                      ! resistance
  REAL, POINTER, DIMENSION(:) :: XDATA_CV               ! vegetation thermal inertia coefficient  (K m2/J)
  REAL, POINTER, DIMENSION(:) :: XDATA_RGL              ! maximum solar radiation
!                                                      ! usable in photosynthesis                (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XDATA_ROOTFRAC       ! root fraction profile ('DIF' option)
!
!-------------------------------------------------------------------------------
!
! - vegetation: Ags parameters ('AGS', 'LAI', 'AST', 'LST', 'NIT', 'NCB' options)
!
  REAL, POINTER, DIMENSION(:)    :: XDATA_BSLAI        ! ratio d(biomass)/d(lai)                 (kg/m2)
  REAL, POINTER, DIMENSION(:)    :: XDATA_LAIMIN       ! minimum LAI (Leaf Area Index)           (m2/m2)
  REAL, POINTER, DIMENSION(:)    :: XDATA_SEFOLD       ! e-folding time for senescence           (s)
  REAL, POINTER, DIMENSION(:)    :: XDATA_H_TREE       ! height of trees                         (m)
  REAL, POINTER, DIMENSION(:)    :: XDATA_GMES         ! mesophyll conductance                   (m s-1)
  REAL, POINTER, DIMENSION(:)    :: XDATA_RE25         ! Ecosystem respiration parameter         (kg m2 s-1)
!
!-------------------------------------------------------------------------------
!
! - vegetation: Ags Stress parameters ('AST', 'LST', 'NIT', 'NCB' options)
!
  LOGICAL, POINTER, DIMENSION(:) :: LDATA_STRESS       ! vegetation response type to water
!                                                     ! stress (true:defensive false:offensive) (-)
  REAL, POINTER, DIMENSION(:)    :: XDATA_F2I          ! critical normilized soil water 
!                                                     ! content for stress parameterisation
  REAL, POINTER, DIMENSION(:)    :: XDATA_GC           ! cuticular conductance                   (m s-1)
  REAL, POINTER, DIMENSION(:)    :: XDATA_DMAX         ! maximum air saturation deficit
!                                                     ! tolerate by vegetation                  (kg/kg)
!
  REAL, POINTER, DIMENSION(:)    :: XDATA_BSLAI_ST     ! ratio d(biomass)/d(lai)                 (kg/m2)
  REAL, POINTER, DIMENSION(:)    :: XDATA_SEFOLD_ST    ! e-folding time for senescence           (s)
  REAL, POINTER, DIMENSION(:)    :: XDATA_GMES_ST      ! mesophyll conductance                   (m s-1)
  REAL, POINTER, DIMENSION(:)    :: XDATA_GC_ST        ! cuticular conductance                   (m s-1)
  REAL, POINTER, DIMENSION(:)    :: XDATA_DMAX_ST      ! maximum air saturation deficit
!-------------------------------------------------------------------------------
!
! - vegetation: Ags Nitrogen-model parameters ('NIT', 'NCB' option)
!
  REAL, POINTER, DIMENSION(:)    :: XDATA_CE_NITRO       ! leaf aera ratio sensitivity to 
!                                                       ! nitrogen concentration                (m2/kg)
  REAL, POINTER, DIMENSION(:)    :: XDATA_CF_NITRO       ! lethal minimum value of leaf area
!                                                       ! ratio                                 (m2/kg)
  REAL, POINTER, DIMENSION(:)    :: XDATA_CNA_NITRO      ! nitrogen concentration of active 
!                                                       ! biomass                               (kg/kg)
!
!-------------------------------------------------------------------------------
!
! - soil: primary parameters
!
  REAL, POINTER, DIMENSION(:,:)  :: XDATA_DG            ! soil layer thicknesses                  (m)
!                                                      ! NOTE: in Force-Restore mode, the 
!                                                      ! uppermost layer thickness is superficial
!                                                      ! and is only explicitly used for soil 
!                                                      ! water phase changes                     (m)
!
  REAL, POINTER,DIMENSION(:)     :: XDATA_DICE       ! depth of the soil column for the calculation
!                                                        of the frozen soil fraction (m)
!
! - bare soil albedo
!
  REAL, POINTER, DIMENSION(:)   :: XDATA_ALBNIR_SOIL      ! soil near-infra-red albedo        (-)
  REAL, POINTER, DIMENSION(:)   :: XDATA_ALBVIS_SOIL      ! soil visible albedo               (-)
  REAL, POINTER, DIMENSION(:)   :: XDATA_ALBUV_SOIL       ! soil UV albedo                    (-)
  REAL, POINTER, DIMENSION(:)   :: XDATA_ALBNIR_DRY       ! dry soil near-infra-red albedo    (-)
  REAL, POINTER, DIMENSION(:)   :: XDATA_ALBVIS_DRY       ! dry soil visible albedo           (-)
  REAL, POINTER, DIMENSION(:)   :: XDATA_ALBUV_DRY        ! dry soil UV albedo                (-)
  REAL, POINTER, DIMENSION(:)   :: XDATA_ALBNIR_WET       ! wet soil near-infra-red albedo    (-)
  REAL, POINTER, DIMENSION(:)   :: XDATA_ALBVIS_WET       ! wet soil visible albedo           (-)
  REAL, POINTER, DIMENSION(:)   :: XDATA_ALBUV_WET        ! wet soil UV albedo                (-)
!
!-------------------------------------------------------------------------------
!
!- Vegetation: Ags Prognostic (YPHOTO = ('LAI', 'LST', 'NIT', or 'NCB') or prescribed (YPHOTO='NON', 'AGS' or 'AST')
!
  REAL, POINTER, DIMENSION(:,:)     :: XDATA_LAI          ! Leaf Area Index                         (m2/m2)
!
!-------------------------------------------------------------------------------
!

END TYPE DATA_TEB_GARDEN_t

TYPE(DATA_TEB_GARDEN_t), ALLOCATABLE, TARGET, SAVE :: DATA_TEB_GARDEN_MODEL(:)

INTEGER, POINTER :: NTIME=>NULL()
!$OMP THREADPRIVATE(NTIME)
REAL, POINTER, DIMENSION(:)      :: XDATA_FRAC_HVEG=>NULL()
!$OMP THREADPRIVATE(XDATA_FRAC_HVEG)
REAL, POINTER, DIMENSION(:)      :: XDATA_FRAC_LVEG=>NULL()
!$OMP THREADPRIVATE(XDATA_FRAC_LVEG)
REAL, POINTER, DIMENSION(:)      :: XDATA_FRAC_NVEG=>NULL()
!$OMP THREADPRIVATE(XDATA_FRAC_NVEG)
REAL, POINTER, DIMENSION(:,:)    :: XDATA_LAI_HVEG=>NULL()
!$OMP THREADPRIVATE(XDATA_LAI_HVEG)
REAL, POINTER, DIMENSION(:,:)    :: XDATA_LAI_LVEG=>NULL()
!$OMP THREADPRIVATE(XDATA_LAI_LVEG)
REAL, POINTER, DIMENSION(:)      :: XDATA_H_HVEG=>NULL()
!$OMP THREADPRIVATE(XDATA_H_HVEG)
REAL, POINTER, DIMENSION(:,:)    :: XDATA_VEGTYPE=>NULL()
!$OMP THREADPRIVATE(XDATA_VEGTYPE)
REAL, POINTER, DIMENSION(:) :: XDATA_Z0_O_Z0H=>NULL()
!$OMP THREADPRIVATE(XDATA_Z0_O_Z0H)
REAL, POINTER, DIMENSION(:,:) :: XDATA_EMIS=>NULL()
!$OMP THREADPRIVATE(XDATA_EMIS)
REAL, POINTER, DIMENSION(:,:) :: XDATA_Z0=>NULL()
!$OMP THREADPRIVATE(XDATA_Z0)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBNIR_VEG=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBNIR_VEG)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBVIS_VEG=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBVIS_VEG)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBUV_VEG=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBUV_VEG)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBNIR_SOIL=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBNIR_SOIL)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBVIS_SOIL=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBVIS_SOIL)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBUV_SOIL=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBUV_SOIL)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBNIR_DRY=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBNIR_DRY)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBVIS_DRY=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBVIS_DRY)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBUV_DRY=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBUV_DRY)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBNIR_WET=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBNIR_WET)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBVIS_WET=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBVIS_WET)
REAL, POINTER, DIMENSION(:) :: XDATA_ALBUV_WET=>NULL()
!$OMP THREADPRIVATE(XDATA_ALBUV_WET)
REAL, POINTER, DIMENSION(:,:) :: XDATA_VEG=>NULL()
!$OMP THREADPRIVATE(XDATA_VEG)
REAL, POINTER, DIMENSION(:) :: XDATA_WRMAX_CF=>NULL()
!$OMP THREADPRIVATE(XDATA_WRMAX_CF)
REAL, POINTER, DIMENSION(:) :: XDATA_RSMIN=>NULL()
!$OMP THREADPRIVATE(XDATA_RSMIN)
REAL, POINTER, DIMENSION(:) :: XDATA_GAMMA=>NULL()
!$OMP THREADPRIVATE(XDATA_GAMMA)
REAL, POINTER, DIMENSION(:) :: XDATA_CV=>NULL()
!$OMP THREADPRIVATE(XDATA_CV)
REAL, POINTER, DIMENSION(:) :: XDATA_RGL=>NULL()
!$OMP THREADPRIVATE(XDATA_RGL)
REAL, POINTER, DIMENSION(:,:) :: XDATA_ROOTFRAC=>NULL()
!$OMP THREADPRIVATE(XDATA_ROOTFRAC)
REAL, POINTER, DIMENSION(:)    :: XDATA_BSLAI=>NULL()
!$OMP THREADPRIVATE(XDATA_BSLAI)
REAL, POINTER, DIMENSION(:)    :: XDATA_LAIMIN=>NULL()
!$OMP THREADPRIVATE(XDATA_LAIMIN)
REAL, POINTER, DIMENSION(:)    :: XDATA_SEFOLD=>NULL()
!$OMP THREADPRIVATE(XDATA_SEFOLD)
REAL, POINTER, DIMENSION(:)    :: XDATA_H_TREE=>NULL()
!$OMP THREADPRIVATE(XDATA_H_TREE)
REAL, POINTER, DIMENSION(:)    :: XDATA_GMES=>NULL()
!$OMP THREADPRIVATE(XDATA_GMES)
REAL, POINTER, DIMENSION(:)    :: XDATA_RE25=>NULL()
!$OMP THREADPRIVATE(XDATA_RE25)
LOGICAL, POINTER, DIMENSION(:) :: LDATA_STRESS=>NULL()
!$OMP THREADPRIVATE(LDATA_STRESS)
REAL, POINTER, DIMENSION(:)    :: XDATA_F2I=>NULL()
!$OMP THREADPRIVATE(XDATA_F2I)
REAL, POINTER, DIMENSION(:)    :: XDATA_GC=>NULL()
!$OMP THREADPRIVATE(XDATA_GC)
REAL, POINTER, DIMENSION(:)    :: XDATA_DMAX=>NULL()
!$OMP THREADPRIVATE(XDATA_DMAX)
REAL, POINTER, DIMENSION(:)    :: XDATA_BSLAI_ST=>NULL()
!$OMP THREADPRIVATE(XDATA_BSLAI_ST)
REAL, POINTER, DIMENSION(:)    :: XDATA_SEFOLD_ST=>NULL()
!$OMP THREADPRIVATE(XDATA_SEFOLD_ST)
REAL, POINTER, DIMENSION(:)    :: XDATA_GMES_ST=>NULL()
!$OMP THREADPRIVATE(XDATA_GMES_ST)
REAL, POINTER, DIMENSION(:)    :: XDATA_GC_ST=>NULL()
!$OMP THREADPRIVATE(XDATA_GC_ST)
REAL, POINTER, DIMENSION(:)    :: XDATA_DMAX_ST=>NULL()
!$OMP THREADPRIVATE(XDATA_DMAX_ST)
REAL, POINTER, DIMENSION(:)    :: XDATA_CE_NITRO=>NULL()
!$OMP THREADPRIVATE(XDATA_CE_NITRO)
REAL, POINTER, DIMENSION(:)    :: XDATA_CF_NITRO=>NULL()
!$OMP THREADPRIVATE(XDATA_CF_NITRO)
REAL, POINTER, DIMENSION(:)    :: XDATA_CNA_NITRO=>NULL()
!$OMP THREADPRIVATE(XDATA_CNA_NITRO)
REAL, POINTER, DIMENSION(:,:)  :: XDATA_DG=>NULL()
!$OMP THREADPRIVATE(XDATA_DG)
REAL, POINTER, DIMENSION(:)     :: XDATA_DICE=>NULL()
!$OMP THREADPRIVATE(XDATA_DICE)
REAL, POINTER, DIMENSION(:,:)     :: XDATA_LAI=>NULL()
!$OMP THREADPRIVATE(XDATA_LAI)

CONTAINS

SUBROUTINE DATA_TEB_GARDEN_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_FRAC_HVEG=>XDATA_FRAC_HVEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_FRAC_LVEG=>XDATA_FRAC_LVEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_FRAC_NVEG=>XDATA_FRAC_NVEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_LAI_HVEG=>XDATA_LAI_HVEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_LAI_LVEG=>XDATA_LAI_LVEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_H_HVEG=>XDATA_H_HVEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_VEGTYPE=>XDATA_VEGTYPE
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_Z0_O_Z0H=>XDATA_Z0_O_Z0H
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_EMIS=>XDATA_EMIS
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_Z0=>XDATA_Z0
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBNIR_VEG=>XDATA_ALBNIR_VEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBVIS_VEG=>XDATA_ALBVIS_VEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBUV_VEG=>XDATA_ALBUV_VEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBNIR_SOIL=>XDATA_ALBNIR_SOIL
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBVIS_SOIL=>XDATA_ALBVIS_SOIL
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBUV_SOIL=>XDATA_ALBUV_SOIL
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBNIR_DRY=>XDATA_ALBNIR_DRY
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBVIS_DRY=>XDATA_ALBVIS_DRY
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBUV_DRY=>XDATA_ALBUV_DRY
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBNIR_WET=>XDATA_ALBNIR_WET
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBVIS_WET=>XDATA_ALBVIS_WET
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ALBUV_WET=>XDATA_ALBUV_WET
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_VEG=>XDATA_VEG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_WRMAX_CF=>XDATA_WRMAX_CF
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_RSMIN=>XDATA_RSMIN
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_GAMMA=>XDATA_GAMMA
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_CV=>XDATA_CV
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_RGL=>XDATA_RGL
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_ROOTFRAC=>XDATA_ROOTFRAC
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_BSLAI=>XDATA_BSLAI
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_LAIMIN=>XDATA_LAIMIN
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_SEFOLD=>XDATA_SEFOLD
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_H_TREE=>XDATA_H_TREE
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_GMES=>XDATA_GMES
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_RE25=>XDATA_RE25
DATA_TEB_GARDEN_MODEL(KFROM)%LDATA_STRESS=>LDATA_STRESS
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_F2I=>XDATA_F2I
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_GC=>XDATA_GC
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_DMAX=>XDATA_DMAX
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_BSLAI_ST=>XDATA_BSLAI_ST
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_SEFOLD_ST=>XDATA_SEFOLD_ST
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_GMES_ST=>XDATA_GMES_ST
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_GC_ST=>XDATA_GC_ST
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_DMAX_ST=>XDATA_DMAX_ST
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_CE_NITRO=>XDATA_CE_NITRO
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_CF_NITRO=>XDATA_CF_NITRO
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_CNA_NITRO=>XDATA_CNA_NITRO
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_DG=>XDATA_DG
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_DICE=>XDATA_DICE
DATA_TEB_GARDEN_MODEL(KFROM)%XDATA_LAI=>XDATA_LAI
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DATA_TEB_GARDEN_N:DATA_TEB_GARDEN_GOTO_MODEL',0,ZHOOK_HANDLE)
NTIME=>DATA_TEB_GARDEN_MODEL(KTO)%NTIME
XDATA_FRAC_HVEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_FRAC_HVEG
XDATA_FRAC_LVEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_FRAC_LVEG
XDATA_FRAC_NVEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_FRAC_NVEG
XDATA_LAI_HVEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_LAI_HVEG
XDATA_LAI_LVEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_LAI_LVEG
XDATA_H_HVEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_H_HVEG
XDATA_VEGTYPE=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_VEGTYPE
XDATA_Z0_O_Z0H=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_Z0_O_Z0H
XDATA_EMIS=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_EMIS
XDATA_Z0=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_Z0
XDATA_ALBNIR_VEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBNIR_VEG
XDATA_ALBVIS_VEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBVIS_VEG
XDATA_ALBUV_VEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBUV_VEG
XDATA_ALBNIR_SOIL=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBNIR_SOIL
XDATA_ALBVIS_SOIL=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBVIS_SOIL
XDATA_ALBUV_SOIL=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBUV_SOIL
XDATA_ALBNIR_DRY=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBNIR_DRY
XDATA_ALBVIS_DRY=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBVIS_DRY
XDATA_ALBUV_DRY=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBUV_DRY
XDATA_ALBNIR_WET=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBNIR_WET
XDATA_ALBVIS_WET=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBVIS_WET
XDATA_ALBUV_WET=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ALBUV_WET
XDATA_VEG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_VEG
XDATA_WRMAX_CF=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_WRMAX_CF
XDATA_RSMIN=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_RSMIN
XDATA_GAMMA=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_GAMMA
XDATA_CV=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_CV
XDATA_RGL=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_RGL
XDATA_ROOTFRAC=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_ROOTFRAC
XDATA_BSLAI=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_BSLAI
XDATA_LAIMIN=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_LAIMIN
XDATA_SEFOLD=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_SEFOLD
XDATA_H_TREE=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_H_TREE
XDATA_GMES=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_GMES
XDATA_RE25=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_RE25
LDATA_STRESS=>DATA_TEB_GARDEN_MODEL(KTO)%LDATA_STRESS
XDATA_F2I=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_F2I
XDATA_GC=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_GC
XDATA_DMAX=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_DMAX
XDATA_BSLAI_ST=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_BSLAI_ST
XDATA_SEFOLD_ST=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_SEFOLD_ST
XDATA_GMES_ST=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_GMES_ST
XDATA_GC_ST=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_GC_ST
XDATA_DMAX_ST=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_DMAX_ST
XDATA_CE_NITRO=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_CE_NITRO
XDATA_CF_NITRO=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_CF_NITRO
XDATA_CNA_NITRO=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_CNA_NITRO
XDATA_DG=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_DG
XDATA_DICE=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_DICE
XDATA_LAI=>DATA_TEB_GARDEN_MODEL(KTO)%XDATA_LAI
IF (LHOOK) CALL DR_HOOK('MODD_DATA_TEB_GARDEN_N:DATA_TEB_GARDEN_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DATA_TEB_GARDEN_GOTO_MODEL

SUBROUTINE DATA_TEB_GARDEN_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TEB_GARDEN_N:DATA_TEB_GARDEN_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DATA_TEB_GARDEN_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_FRAC_HVEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_FRAC_LVEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_FRAC_NVEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_LAI_HVEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_LAI_LVEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_H_HVEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_VEGTYPE)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_Z0_O_Z0H)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_EMIS)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_Z0)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBNIR_VEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBVIS_VEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBUV_VEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_VEG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_WRMAX_CF)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_RSMIN)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_GAMMA)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_CV)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_RGL)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ROOTFRAC)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_BSLAI)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_LAIMIN)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_SEFOLD)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_H_TREE)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_GMES)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_RE25)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%LDATA_STRESS)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_F2I)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_GC)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_DMAX)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_BSLAI_ST)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_SEFOLD_ST)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_GMES_ST)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_GC_ST)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_DMAX_ST)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_CE_NITRO)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_CF_NITRO)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_CNA_NITRO)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_DG)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_DICE)  
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBNIR_SOIL)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBVIS_SOIL)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBUV_SOIL)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBNIR_DRY)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBVIS_DRY)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBUV_DRY)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBNIR_WET)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBVIS_WET)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_ALBUV_WET)
  NULLIFY(DATA_TEB_GARDEN_MODEL(J)%XDATA_LAI)
ENDDO
DATA_TEB_GARDEN_MODEL(:)%NTIME=0
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TEB_GARDEN_N:DATA_TEB_GARDEN_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DATA_TEB_GARDEN_ALLOC

SUBROUTINE DATA_TEB_GARDEN_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TEB_GARDEN_N:DATA_TEB_GARDEN_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DATA_TEB_GARDEN_MODEL)) DEALLOCATE(DATA_TEB_GARDEN_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TEB_GARDEN_N:DATA_TEB_GARDEN_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DATA_TEB_GARDEN_DEALLO

END MODULE MODD_DATA_TEB_GARDEN_n
