!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ############################
      MODULE MODD_DIAG_UTCI_TEB_n
!     ############################
!
!!****  *MODD_DIAG_UTCI_TEB - declaration of confort from TEB scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	P. Le Moigne   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/10/04
!
!
!*       0.   DECLARATIONS
!             ------------
!
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DIAG_UTCI_TEB_t
!------------------------------------------------------------------------------
!
  LOGICAL :: LUTCI                               ! flag to compute UTCI quantities
  REAL, POINTER, DIMENSION(:)   :: XUTCI_IN      ! UTCI for person indoor
  REAL, POINTER, DIMENSION(:)   :: XUTCI_OUTSUN  ! UTCI for person outdoor at sun
  REAL, POINTER, DIMENSION(:)   :: XUTCI_OUTSHADE! UTCI for person outdoor at shade
  REAL, POINTER, DIMENSION(:)   :: XTRAD_SUN! Mean radiant temperature seen by person at sun (K)
  REAL, POINTER, DIMENSION(:)   :: XTRAD_SHADE! Mean radiant temperature seen by person in shade (K)
!
END TYPE DIAG_UTCI_TEB_t
!

TYPE(DIAG_UTCI_TEB_t), ALLOCATABLE, TARGET, SAVE :: DIAG_UTCI_TEB_MODEL(:)

LOGICAL, POINTER :: LUTCI=>NULL()
!$OMP THREADPRIVATE(LUTCI)
REAL, POINTER, DIMENSION(:)   :: XUTCI_IN=>NULL()
!$OMP THREADPRIVATE(XUTCI_IN)
REAL, POINTER, DIMENSION(:)   :: XUTCI_OUTSUN=>NULL()
!$OMP THREADPRIVATE(XUTCI_OUTSUN)
REAL, POINTER, DIMENSION(:)   :: XUTCI_OUTSHADE=>NULL()
!$OMP THREADPRIVATE(XUTCI_OUTSHADE)
REAL, POINTER, DIMENSION(:)   :: XTRAD_SUN=>NULL()
!$OMP THREADPRIVATE(XTRAD_SUN)
REAL, POINTER, DIMENSION(:)   :: XTRAD_SHADE=>NULL()
!$OMP THREADPRIVATE(XTRAD_SHADE)

CONTAINS

SUBROUTINE DIAG_UTCI_TEB_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_UTCI_TEB_N:DIAG_UTCI_TEB_GOTO_MODEL',0,ZHOOK_HANDLE)
DIAG_UTCI_TEB_MODEL(KFROM)%XUTCI_IN=>XUTCI_IN
DIAG_UTCI_TEB_MODEL(KFROM)%XUTCI_OUTSUN=>XUTCI_OUTSUN
DIAG_UTCI_TEB_MODEL(KFROM)%XUTCI_OUTSHADE=>XUTCI_OUTSHADE
DIAG_UTCI_TEB_MODEL(KFROM)%XTRAD_SUN=>XTRAD_SUN
DIAG_UTCI_TEB_MODEL(KFROM)%XTRAD_SHADE=>XTRAD_SHADE
!
LUTCI => DIAG_UTCI_TEB_MODEL(KTO)%LUTCI
XUTCI_IN => DIAG_UTCI_TEB_MODEL(KTO)%XUTCI_IN
XUTCI_OUTSUN => DIAG_UTCI_TEB_MODEL(KTO)%XUTCI_OUTSUN
XUTCI_OUTSHADE => DIAG_UTCI_TEB_MODEL(KTO)%XUTCI_OUTSHADE
XTRAD_SUN => DIAG_UTCI_TEB_MODEL(KTO)%XTRAD_SUN
XTRAD_SHADE => DIAG_UTCI_TEB_MODEL(KTO)%XTRAD_SHADE
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_UTCI_TEB_N:DIAG_UTCI_TEB_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DIAG_UTCI_TEB_GOTO_MODEL

SUBROUTINE DIAG_UTCI_TEB_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_UTCI_TEB_N:DIAG_UTCI_TEB_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_UTCI_TEB_MODEL(KMODEL))
DIAG_UTCI_TEB_MODEL(:)%LUTCI=.FALSE.
DO J=1,KMODEL
  NULLIFY(DIAG_UTCI_TEB_MODEL(J)%XUTCI_IN)
  NULLIFY(DIAG_UTCI_TEB_MODEL(J)%XUTCI_OUTSUN)
  NULLIFY(DIAG_UTCI_TEB_MODEL(J)%XUTCI_OUTSHADE)
  NULLIFY(DIAG_UTCI_TEB_MODEL(J)%XTRAD_SUN)
  NULLIFY(DIAG_UTCI_TEB_MODEL(J)%XTRAD_SHADE)
END DO
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_UTCI_TEB_N:DIAG_UTCI_TEB_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_UTCI_TEB_ALLOC

SUBROUTINE DIAG_UTCI_TEB_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_UTCI_TEB_OTPIONS_N:DIAG_UTCI_TEB_OTPIONS_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_UTCI_TEB_MODEL)) DEALLOCATE(DIAG_UTCI_TEB_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_UTCI_TEB_OTPIONS_N:DIAG_UTCI_TEB_OTPIONS_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_UTCI_TEB_DEALLO


END MODULE MODD_DIAG_UTCI_TEB_n
