!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ##################
      MODULE MODD_OCEAN_REL_n
!     ##################
!
!     Author : P. Peyrille
!!    Date: 01/2012
!! 
!!  declaration of relaxation field, flag for ocean model
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
TYPE OCEAN_REL_t
!
 REAL                :: XTAU_REL   ! Time of relaxation  (s)
!
 REAL                :: XQCORR     ! correction coefficient  for surface fluxes  (BArnier et al. 1998) typcial val: 100.
!
 LOGICAL                            :: LREL_CUR   ! flag for  relxation on current
 LOGICAL                            :: LREL_TS    ! flag for relaxation on ocean temperature
 LOGICAL                            :: LFLUX_NULL ! Flag for testing zero  incoming flux at the surface 
!
 LOGICAL                            :: LFLX_CORR  ! Flag for  flux  correction 
 LOGICAL                            :: LDIAPYCNAL ! Flag for diapycnal mixing activation
!
 REAL, POINTER, DIMENSION(:,:) :: XSEAU_REL  ! ref. U current  profile (C)
 REAL, POINTER, DIMENSION(:,:) :: XSEAV_REL  ! ref. V current  profile (C)
 REAL, POINTER, DIMENSION(:,:) :: XSEAT_REL  ! ref. temperature  profile (C)
 REAL, POINTER, DIMENSION(:,:) :: XSEAS_REL  ! ---- salinity   ---------- (%)
!
END TYPE OCEAN_REL_t
!
TYPE(OCEAN_REL_t), ALLOCATABLE, TARGET, SAVE :: OCEAN_REL_MODEL(:)
!
REAL, POINTER :: XTAU_REL=>NULL() 
!$OMP THREADPRIVATE(XTAU_REL)
REAL, POINTER :: XQCORR=>NULL()
!$OMP THREADPRIVATE(XQCORR)
!
LOGICAL, POINTER :: LREL_CUR=>NULL()
!$OMP THREADPRIVATE(LREL_CUR)
LOGICAL, POINTER :: LREL_TS=>NULL()
!$OMP THREADPRIVATE(LREL_TS)
LOGICAL, POINTER :: LFLUX_NULL=>NULL()
!$OMP THREADPRIVATE(LFLUX_NULL)
!
LOGICAL, POINTER :: LFLX_CORR=>NULL()
!$OMP THREADPRIVATE(LFLX_CORR)
LOGICAL, POINTER :: LDIAPYCNAL=>NULL()
!$OMP THREADPRIVATE(LDIAPYCNAL)
!
REAL, POINTER, DIMENSION(:,:) :: XSEAU_REL=>NULL()
!$OMP THREADPRIVATE(XSEAU_REL)
REAL, POINTER, DIMENSION(:,:) :: XSEAV_REL=>NULL()
!$OMP THREADPRIVATE(XSEAV_REL)
REAL, POINTER, DIMENSION(:,:) :: XSEAT_REL=>NULL()
!$OMP THREADPRIVATE(XSEAT_REL)
REAL, POINTER, DIMENSION(:,:) :: XSEAS_REL=>NULL()
!$OMP THREADPRIVATE(XSEAS_REL)
!
CONTAINS
!
SUBROUTINE OCEAN_REL_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('MODD_OCEAN_REL_N:OCEAN_REL_GOTO_MODEL',0,ZHOOK_HANDLE)
!
IF (LKFROM) THEN
OCEAN_REL_MODEL(KFROM)%XSEAT_REL=>XSEAT_REL
OCEAN_REL_MODEL(KFROM)%XSEAS_REL=>XSEAS_REL
OCEAN_REL_MODEL(KFROM)%XSEAU_REL=>XSEAU_REL
OCEAN_REL_MODEL(KFROM)%XSEAV_REL=>XSEAV_REL
ENDIF
!
XTAU_REL=>OCEAN_REL_MODEL(KTO)%XTAU_REL
XQCORR=>OCEAN_REL_MODEL(KTO)%XQCORR
LREL_CUR=>OCEAN_REL_MODEL(KTO)%LREL_CUR
LREL_TS=>OCEAN_REL_MODEL(KTO)%LREL_TS
LFLUX_NULL=>OCEAN_REL_MODEL(KTO)%LFLUX_NULL
LFLX_CORR=>OCEAN_REL_MODEL(KTO)%LFLX_CORR
LDIAPYCNAL=>OCEAN_REL_MODEL(KTO)%LDIAPYCNAL
XSEAT_REL=>OCEAN_REL_MODEL(KTO)%XSEAT_REL
XSEAS_REL=>OCEAN_REL_MODEL(KTO)%XSEAS_REL
XSEAU_REL=>OCEAN_REL_MODEL(KTO)%XSEAU_REL
XSEAV_REL=>OCEAN_REL_MODEL(KTO)%XSEAV_REL
!
IF (LHOOK) CALL DR_HOOK('MODD_OCEAN_REL_N:OCEAN_REL_GOTO_MODEL',1,ZHOOK_HANDLE)
!
END SUBROUTINE OCEAN_REL_GOTO_MODEL
!
SUBROUTINE OCEAN_REL_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_OCEAN_REL_N:OCEAN_REL_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(OCEAN_REL_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(OCEAN_REL_MODEL(J)%XSEAT_REL)
  NULLIFY(OCEAN_REL_MODEL(J)%XSEAS_REL)
  NULLIFY(OCEAN_REL_MODEL(J)%XSEAU_REL)
  NULLIFY(OCEAN_REL_MODEL(J)%XSEAV_REL)
ENDDO
OCEAN_REL_MODEL(:)%XTAU_REL=0.
OCEAN_REL_MODEL(:)%XQCORR=0.
OCEAN_REL_MODEL(:)%LREL_CUR=.FALSE.
OCEAN_REL_MODEL(:)%LREL_TS=.FALSE.
OCEAN_REL_MODEL(:)%LFLUX_NULL=.FALSE.
OCEAN_REL_MODEL(:)%LFLX_CORR=.FALSE.
OCEAN_REL_MODEL(:)%LDIAPYCNAL=.FALSE.
!
IF (LHOOK) CALL DR_HOOK("MODD_OCEAN_REL_N:OCEAN_REL_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE OCEAN_REL_ALLOC
!
SUBROUTINE OCEAN_REL_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_OCEAN_REL_N:OCEAN_DEALLO_REL",0,ZHOOK_HANDLE)
IF (ALLOCATED(OCEAN_REL_MODEL)) DEALLOCATE(OCEAN_REL_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_OCEAN_REL_N:OCEAN_REL_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE OCEAN_REL_DEALLO
!
END MODULE MODD_OCEAN_REL_n
